import { PointModel } from "../../point-component/models/point-model";
import { ELEMENTS_CLASS } from "../../../core/enums/elements-class-enum";
import { DrawerAggregator } from "../../../core/drawers/drawer-aggregator";
import { GeometricObject } from "../../../core/models/objects/geometric-object";
import { setWith } from "lodash";

export class IntersectionModel extends PointModel {
  /**
   * 
   * @param {GeometricObject} r Geometric Object 
   * @param {GeometricObject} s Geometric Object
   * @param {boolean} visible Visiblity of Object 
   * @param {number} index Index position of Object ex (1)
   * @param {id} id indentity of intersection ex: 0
   */
  constructor(posX, posY, label, r, s, visible, index, id) {
    super(posX, posY, label, id);
    this.r = r;
    this.s = s;
    super.setClass(ELEMENTS_CLASS.INTERSECTION_POINT);
    this.visible = visible;
    this.index = index;
    this.color = -65536;
    this.definitions = this.getDefinitions();
  }

  /**
   * Update properties of this Intersection
   * @param {DrawerAggregator} aggregator Drawer Aggregator 
   * @param {event} event 
  */
  update(aggregator, event) {
    const intersections = this.r.getIntersection(this.s);
    if (intersections.length == 1) {
      this.visible = true;
      const intersection = intersections[0];
      this.posX = parseFloat(intersection.posX.toFixed(2));
      this.posY = parseFloat(intersection.posY.toFixed(2));
      // this.visible = intersection.visible;
      if (!this.r.insideSegment(this.posX, this.posY)) {
        this.posX = Number.MAX_SAFE_INTEGER;
        this.posY = Number.MAX_SAFE_INTEGER;
        this.visible = false;
        this.definitions = this.getDefinitions();
        return;
      }
      if (!this.s.insideSegment(this.posX, this.posY)) {
        this.posX = Number.MAX_SAFE_INTEGER;
        this.posY = Number.MAX_SAFE_INTEGER;
        this.visible = false;
        this.definitions = this.getDefinitions();
        return;
      }
      return;
    }
    if (intersections.length > 1) {
      for (let index = 0; index < intersections.length; index++) {
        const intersection = intersections[index];
        if (this.index == index) {
          this.posX = parseFloat(intersection.posX.toFixed(2));
          this.posY = parseFloat(intersection.posY.toFixed(2));
          this.visible = intersection.visible;
        }
      }
    }
    this.definitions = this.getDefinitions();
  }

  getDefinitions() {
    return [{ id: this.r.id }, { id: this.s.id }, { id: this.index + 1 }, { id: this.visible ? 1 : 0 }];
  }

  bind(posX, posY, label, r, s, visible, index) {
    super.bind(posX, posY, label);
    this.r = r;
    this.s = s;
    this.visible = visible;
    this.index = index;
    this.color = -65536;
    this.definitions = this.getDefinitions();
    super.setClass(ELEMENTS_CLASS.INTERSECTION_POINT);
  }

  /**
   * Create new Intersection By Line of Script .geo
   * @param {Map} map JavaScript Map
   * @param {[]} list List of Generic Objects
   */
  static do(map, list) {
    const id = map.get("id");
    const rId = map.get("param")[0];
    const sId = map.get("param")[1];
    const index = map.get("param")[2] - 1;
    const visible = map.get("param")[5] == 1;
    const label = map.get("label")[0];
    const r = list.find(x => x.id == rId);
    const s = list.find(x => x.id == sId);

    const intersections = r.getIntersection(s);
    if (intersections.length == 1) {
      const i = intersections[0];
      i.bind(i.posX, i.posY, label, r, s, true, index);
      return i;
    } else {
      const i = intersections.find(x => x.index == index);
      i.bind(i.posX, i.posY, label, r, s, true, index);
      return i;
    }
  }

}
